#include <stdlib.h>
#include <stdio.h>

#include "io.hh"

// generates Sierpinski-fractal
inline void sierpinski2(int offset, int size, int initialSize, int iter, int* b)
{
  int i, j, k;

  if (iter > 0)
  {
    const int newsize = (int) size/3;
    for (i=0; i<newsize; ++i)
    {
      for (j=0; j<newsize; ++j)
      {
        const int index = offset + initialSize * (newsize + j) + (newsize + i);
        b[index] = 1;
      }
    }
    for (k=0; k<9; ++k)
    {
      if (k != 4)
      {
        const int xo = k % 3;
        const int yo = k / 3;
        const int newOffset = offset + yo * newsize * initialSize + xo * newsize;
        sierpinski2(newOffset, newsize, initialSize, iter-1, b);
      }
    }
  }
}

// main function
int main(int argc, char** argv)
{
  // read arguments
  if (argc != 3)
  {
    printf("Usage: %s <initial size s> <depth n>\n", argv[0]);
    exit(1);
  }
  int s = atoi(argv[1]);
  int n = atoi(argv[2]);

  // arrays for the binary values of each pixel, initialize white
  int* b = (int*) malloc(s * s * sizeof(int));
  if (NULL == b)
  {
    printf("Could not allocate memory.\n");
    exit(1);
  }

  int i, j;
  for (i=0; i<s; ++i)
    for (j=0; j<s; ++j)
    {
      const int index = i * s + j;
      b[index] = 0;
    }

  // generate ppm-file and output ppm-file
  sierpinski2(0, s, s, n, b);
  char* filename = "sierpinski.pbm";
  pbma_write(filename, s, s, b);

  return 0;
}
